@@ -58,6 +58,25 @@ typedef enum {
         VTE_CURSOR_SHAPE_UNDERLINE
 } VteCursorShape;
 
+/**
+ * VteTextBlinkMode:
+ * @VTE_TEXT_BLINK_NEVER: Do not blink the text.
+ * @VTE_TEXT_BLINK_FOCUSED: Allow blinking text only if the terminal is focused.
+ * @VTE_TEXT_BLINK_UNFOCUSED: Allow blinking text only if the terminal is unfocused.
+ * @VTE_TEXT_BLINK_ALWAYS: Allow blinking text. This is the default.
+ *
+ * An enumerated type which can be used to indicate whether the terminal allows
+ * the text contents to be blinked.
+ *
+ * Since: 0.52
+ */
+typedef enum {
+        VTE_TEXT_BLINK_NEVER     = 0,
+        VTE_TEXT_BLINK_FOCUSED   = 1,
+        VTE_TEXT_BLINK_UNFOCUSED = 2,
+        VTE_TEXT_BLINK_ALWAYS    = 3
+} VteTextBlinkMode;
+
 /**
  * VteEraseBinding:
  * @VTE_ERASE_AUTO: For backspace, attempt to determine the right value from the terminal's IO settings.  For delete, use the control sequence.
@@ -117,6 +136,41 @@ typedef enum {
   VTE_WRITE_DEFAULT = 0
 } VteWriteFlags;
 
+/**
+ * VteRegexError:
+ * @VTE_REGEX_ERROR_INCOMPATIBLE: The PCRE2 library was built without
+ *   Unicode support which is required for VTE
+ * @VTE_REGEX_ERROR_NOT_SUPPORTED: Regexes are not supported because VTE was
+ *   built without PCRE2 support
+ *
+ * An enum type for regex errors. In addition to the values listed above,
+ * any PCRE2 error values may occur.
+ *
+ * Since: 0.46
+ */
+typedef enum {
+        /* Negative values are PCRE2 errors */
+
+        /* VTE specific values */
+        VTE_REGEX_ERROR_INCOMPATIBLE  = G_MAXINT-1,
+        VTE_REGEX_ERROR_NOT_SUPPORTED = G_MAXINT
+} VteRegexError;
+
+/**
+ * VteFormat:
+ * @VTE_FORMAT_TEXT: Export as plain text
+ * @VTE_FORMAT_HTML: Export as HTML formatted text
+ *
+ * An enumeratio type that can be used to specify the format the selection
+ * should be copied to the clipboard in.
+ *
+ * Since: 0.50
+ */
+typedef enum {
+        VTE_FORMAT_TEXT = 1,
+        VTE_FORMAT_HTML = 2
+} VteFormat;
+
 G_END_DECLS
 
 #endif /* __VTE_VTE_ENUMS_H__ */
