@@ -26,12 +26,7 @@
 #include <glib.h>
 #include "buffer.h"
 #include "vteconv.h"
-
-#ifdef VTE_COMPILATION
-#include "vte-private.h"
-#else
-#define VTE_UTF8_BPC                    (6) /* Maximum number of bytes used per UTF-8 character */
-#endif
+#include "vtedefines.hh"
 
 typedef size_t (*convert_func)(GIConv converter,
 			  const guchar **inbuf,
@@ -776,7 +771,7 @@ int
 main (int argc,
       char *argv[])
 {
-        g_test_init (&argc, &argv, NULL);
+        g_test_init (&argc, &argv, nullptr);
 
         g_test_add_func ("/vte/conv/utf8/strlen", test_utf8_strlen);
         g_test_add_func ("/vte/conv/utf8/validate", test_utf8_validate);
