@@ -127,6 +127,10 @@ typedef guint8 _vte_overwrite_counter_t;
 
 /******************************************************************************************/
 
+#ifndef HAVE_EXPLICIT_BZERO
+#define explicit_bzero(s, n) memset((s), 0, (n))
+#endif
+
 #ifndef HAVE_PREAD
 #define pread _pread
 static inline gsize
@@ -830,12 +834,10 @@ _vte_boa_init (VteBoa *boa)
         datum_key.data = key;
         datum_key.size = VTE_CIPHER_KEY_SIZE;
         gnutls_cipher_init(&boa->cipher_hd, VTE_CIPHER_ALGORITHM, &datum_key, NULL);
-        /* FIXME: 738601#c52 the compiler might optimize this away, how to make sure it's erased?
-         * It's on the stack so maybe we can rest assured it'll be overwritten pretty soon. */
-        memset(key, 0, VTE_CIPHER_KEY_SIZE);
+        explicit_bzero(key, VTE_CIPHER_KEY_SIZE);
 
         /* Empty IV. */
-        memset(&boa->iv, 0, sizeof(boa->iv));
+        explicit_bzero(&boa->iv, sizeof(boa->iv));
 #endif
 
         boa->compressBound = _vte_boa_compressBound(VTE_BOA_BLOCKSIZE);
@@ -847,7 +849,7 @@ _vte_boa_finalize (GObject *object)
 #if !defined VTESTREAM_MAIN && defined WITH_GNUTLS
         VteBoa *boa = (VteBoa *) object;
 
-        memset(&boa->iv, 0, sizeof(boa->iv));
+        explicit_bzero(&boa->iv, sizeof(boa->iv));
 
         gnutls_cipher_deinit (boa->cipher_hd);
         gnutls_global_deinit ();
@@ -883,25 +885,24 @@ static gboolean
 _vte_boa_read_with_overwrite_counter (VteBoa *boa, gsize offset, char *data, _vte_overwrite_counter_t *overwrite_counter)
 {
         _vte_block_datalength_t compressed_len;
-        gboolean ret = FALSE;
-        char *buf = (char *)g_malloc(VTE_SNAKE_BLOCKSIZE);
+        char *buf = g_newa(char, VTE_SNAKE_BLOCKSIZE);
 
         g_assert_cmpuint (offset % VTE_BOA_BLOCKSIZE, ==, 0);
 
         /* Read */
         if (G_UNLIKELY (!_vte_snake_read (&boa->parent, OFFSET_BOA_TO_SNAKE(offset), buf)))
-                goto out;
+                return FALSE;
 
         compressed_len = *((_vte_block_datalength_t *) buf);
         *overwrite_counter = *((_vte_overwrite_counter_t *) (buf + VTE_BLOCK_DATALENGTH_SIZE));
 
         /* We could have read an empty block due to a previous disk full. Treat that as an error too. Perform other sanity checks. */
         if (G_UNLIKELY (compressed_len <= 0 || compressed_len > VTE_BOA_BLOCKSIZE || *overwrite_counter <= 0))
-                goto out;
+                return FALSE;
 
         /* Decrypt, bail out on tag mismatch */
         if (G_UNLIKELY (!_vte_boa_decrypt (boa, offset, *overwrite_counter, buf + VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE, compressed_len)))
-                goto out;
+                return FALSE;
 
         /* Uncompress, or copy if wasn't compressable */
         if (G_LIKELY (data != NULL)) {
@@ -913,11 +914,7 @@ _vte_boa_read_with_overwrite_counter (VteBoa *boa, gsize offset, char *data, _vt
                         g_assert_cmpuint (uncompressed_len, ==, VTE_BOA_BLOCKSIZE);
                 }
         }
-        ret = TRUE;
-
-out:
-        g_free(buf);
-        return ret;
+        return TRUE;
 }
 
 static gboolean
@@ -934,8 +931,6 @@ _vte_boa_read (VteBoa *boa, gsize offset, char *data)
 static void
 _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
 {
-        _vte_block_datalength_t compressed_len = boa->compressBound;
-
         /* The overwrite counter is 1-based.  This is to make sure that the IV is never 0: 738601#c88,
            to make sure that an empty block (e.g. after a previous write failure) is always invalid,
            and to make unit testing easier */
@@ -943,8 +938,8 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
 
         /* The helper buffer should be large enough to contain a whole snake block,
          * and also large enough to compress data that actually grows bigger during compression. */
-        char *buf = (char *)g_malloc(MAX(VTE_SNAKE_BLOCKSIZE,
-                                         VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE + boa->compressBound));
+        char *buf = g_newa(char, MAX(VTE_SNAKE_BLOCKSIZE,
+                                     VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE + boa->compressBound));
 
         g_assert_cmpuint (offset, >=, boa->tail);
         g_assert_cmpuint (offset, <=, boa->head);
@@ -964,11 +959,13 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
                         _vte_snake_write (&boa->parent, OFFSET_BOA_TO_SNAKE(offset), buf, VTE_SNAKE_BLOCKSIZE);
                         /* Try to punch out from the FS */
                         _vte_snake_write (&boa->parent, OFFSET_BOA_TO_SNAKE(offset), "", 0);
-                        goto out;
+                        return;
                 }
                 overwrite_counter++;
         }
 
+        _vte_block_datalength_t compressed_len;
+
         /* Compress, or copy if uncompressable */
         compressed_len = _vte_boa_compress (buf + VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE, boa->compressBound,
                                             data, VTE_BOA_BLOCKSIZE);
@@ -989,9 +986,6 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
         if (G_LIKELY (offset == boa->head)) {
                 boa->head += VTE_BOA_BLOCKSIZE;
         }
-
-out:
-        g_free(buf);
 }
 
 static void
@@ -1073,7 +1067,6 @@ static void
 _vte_file_stream_init (VteFileStream *stream)
 {
         stream->boa = (VteBoa *)g_object_new (VTE_TYPE_BOA, NULL);
-        _vte_boa_init (stream->boa);
 
         stream->rbuf = (char *)g_malloc(VTE_BOA_BLOCKSIZE);
         stream->wbuf = (char *)g_malloc(VTE_BOA_BLOCKSIZE);
